/**
 * Copyright 2023 Velocity Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

const { createS3Client } = require('./aws-factory');

const initS3Client = ({
  awsRegion,
  awsEndpoint,
  s3Bucket,
  s3PresignedUrlExpiration,
  accessKeyId,
  secretAccessKey,
  isTest,
}) => {
  const s3Client = createS3Client({
    awsRegion,
    awsEndpoint,
    accessKeyId,
    secretAccessKey,
    isTest,
  });

  const getSignedUrl = async ({ key, contentType, metadata }) =>
    s3Client.getSignedUrlPromise('putObject', {
      Bucket: s3Bucket,
      Key: key,
      Expires: s3PresignedUrlExpiration,
      ContentType: contentType,
      Metadata: metadata,
    });

  const getObject = async ({ bucket, key }) => {
    const params = {
      Bucket: bucket,
      Key: key,
    };

    return s3Client.getObject(params).promise();
  };

  const putObject = async ({ bucket, key, body, contentType, metadata }) => {
    const params = {
      Bucket: bucket,
      Key: key,
      Body: body,
      ContentType: contentType,
      Metadata: metadata,
    };

    await s3Client.putObject(params).promise();
  };

  const deleteObject = async ({ bucket, key }) => {
    const params = {
      Bucket: bucket,
      Key: key,
    };
    await s3Client.deleteObject(params).promise();
  };

  return {
    getObject,
    putObject,
    getSignedUrl,
    deleteObject,
  };
};

module.exports = {
  initS3Client,
};
